﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace MVCBugTracker.Models
{
    public class EFBugRepository : IBugRepository
    {
        MVCBUGTRACKEREntities db = new MVCBUGTRACKEREntities();

        public IQueryable<Bug> GetBugsByProject(int projectID)
        {
            return from b in db.Bugs
                   where b.ProjectID == projectID
                   && b.Deleted == false
                   orderby b.LastUpdate descending
                   select b;
        }

        public IQueryable<Bug> GetMyBugs(string UserName)
        {
            return from b in db.Bugs
                   where b.AssignedUserName == UserName
                   && b.Deleted == false
                   orderby b.LastUpdate descending
                   select b;
        }

        public IQueryable<Bug> GetBugsByCategory(int projectID, int categoryID)
        {
            return from b in db.Bugs
                   where b.ProjectID == projectID
                   && b.CategoryID == categoryID
                   && b.Deleted == false
                   orderby b.LastUpdate descending
                   select b;
        }

        public IQueryable<Bug> GetBugsByMilestone(int projectID, int milestoneID)
        {
            return from b in db.Bugs
                   where b.ProjectID == projectID
                   && b.MilestoneID == milestoneID
                   && b.Deleted == false
                   orderby b.LastUpdate descending
                   select b;
        }

        public IQueryable<Bug> GetBugsByPriority(int projectID, int priorityID)
        {
            return from b in db.Bugs
                   where b.ProjectID == projectID
                   && b.PriorityID == priorityID
                   && b.Deleted == false
                   orderby b.LastUpdate descending
                   select b;
        }

        public IQueryable<Bug> GetBugsByStatus(int projectID, int statusID)
        {
            return from b in db.Bugs
                   where b.ProjectID == projectID
                   && b.StatusID == statusID
                   && b.Deleted == false
                   orderby b.LastUpdate descending
                   select b;
        }

        public Bug GetBug(int bugID)
        {
            return db.Bugs.SingleOrDefault(b => b.BugID == bugID);
        }

        public void AddBug(Bug bug)
        {
            db.Bugs.AddObject(bug);
        }

        public void DeleteBug(Bug bug)
        {
            bug.Deleted = true;
        }

        public void Save()
        {
            db.SaveChanges();
        }

        public IQueryable<BugComment> GetBugComments(int bugID)
        {
            return from b in db.BugComments
                   where b.BugID == bugID
                   orderby b.DateCreated ascending
                   select b;
        }

        public BugComment GetBugComment(int commentID)
        {
            return db.BugComments.SingleOrDefault(b => b.CommentID == commentID);
        }

        public void AddBugComment(BugComment comment)
        {
            db.BugComments.AddObject(comment);
        }

        public void DeleteBugComment(BugComment comment)
        {
            db.BugComments.DeleteObject(comment);
        }

        public IQueryable<BugHistory> GetBugHistories(int bugID)
        {
            return from h in db.BugHistories
                   where h.BugID == bugID
                   orderby h.DateCreated descending
                   select h;
        }

        public BugHistory GetBugHistory(int historyID)
        {
            return db.BugHistories.SingleOrDefault(h => h.HistoryID == historyID);
        }

        public void AddBugHistory(BugHistory history)
        {
            db.BugHistories.AddObject(history);
        }

        public void DeleteBugHistory(BugHistory history)
        {
            db.BugHistories.DeleteObject(history);
        }

        public IQueryable<BugNotification> GetBugNotifications(int bugID)
        {
            return from b in db.BugNotifications
                   where b.BugID == bugID
                   select b;
        }

        public BugNotification GetBugNotification(int notificationID)
        {
            return db.BugNotifications.SingleOrDefault(n => n.NotificationID == notificationID);
        }

        public void AddBugNotification(BugNotification notification)
        {
            db.BugNotifications.AddObject(notification);
        }

        public void DeleteBugNotification(BugNotification notification)
        {
            db.BugNotifications.DeleteObject(notification);
        }

        public IQueryable<BugAttachment> GetBugAttachments(int bugID)
        {
            return from a in db.BugAttachments
                   where a.BugID == bugID
                   select a;
        }

        public BugAttachment GetBugAttachment(int attachmentID)
        {
            return db.BugAttachments.SingleOrDefault(a => a.AttachmentID == attachmentID);
        }

        public void AddBugAttachment(BugAttachment attachment)
        {
            db.BugAttachments.AddObject(attachment);
        }

        public void DeleteBugAttachment(BugAttachment attachment)
        {
            db.BugAttachments.DeleteObject(attachment);
        }
    }
}